import random

from helpers import StoreHelper, FolderHelper
import acl

from MAPI import *
from MAPI.Tags import *
from MAPI.Struct import *
from MAPI.Util import *
from MAPI.Util.Generators import GetStores
from MAPI.Util.Iterators import FolderIterator

import licensereq

class Stats:
    class TypedStats:
        def __init__(self):
            self.success = 0
            self.failed = 0
            self.skipped = 0
    
    def __init__(self):
        self.storestats = self.TypedStats()
        self.folderstats = self.TypedStats()
        self.archivestats = self.TypedStats()
        self.accessstats = self.TypedStats()

    def __getattr__(self, attr):
        if attr == 'retval':
            if self.storestats.failed == 0 and self.folderstats.failed == 0 and self.archivestats.failed == 0 and self.accessstats.failed == 0:
                return 0
            return 1
        else:
            raise AttributeError()

    def __str__(self):
        str =  "Successfully processed %u store(s).\n" % self.storestats.success
        str += "Skipped %u unarchived store(s).\n" % self.storestats.skipped
        str += "%u store(s) failed.\n" % self.storestats.failed
        str += "Successfully updated access rights of %u archive(s).\n" % self.accessstats.success
        str += "Skipped %u archive(s).\n" % self.accessstats.skipped
        str += "Failed to update access rights of %u archive(s).\n" % self.accessstats.failed
        return str



class ACLSetter:
    def __init__(self, storehelper):
        self.storehelper = storehelper
        self.success = True

    def __GetIpmSubtree(self, store):
        id = HrGetOneProp(store, PR_IPM_SUBTREE_ENTRYID).Value
        return store.OpenEntry(id, IID_IMAPIFolder, MAPI_MODIFY)

    def __IsIpmSubtree(self, store, folder):
        ipmid = HrGetOneProp(store, PR_IPM_SUBTREE_ENTRYID).Value
        fldid = HrGetOneProp(folder, PR_ENTRYID).Value
        return store.CompareEntryIDs(ipmid, fldid, 0)        

    def UpdateStoreAccess(self, writable):
        global stats        
        for archive in self.storehelper.GetArchives():
            if archive is None:
                stats.accessstats.skipped += 1
                continue

            try:
                archive_acl = acl.create(archive[1])
                if archive_acl.SetRights(self.storehelper.owner, writable):
                    stats.accessstats.success += 1
                else:
                    stats.accessstats.skipped += 1
            except MAPIError, err:
                print "Unhandled MAPI error occurred while updating archive access, hr=0x%08x." % err.hr
                stats.accessstats.failed += 1
                self.success = False
            except:
                print "Unknown error occurred while updating archive access."
                stats.accessstats.failed += 1
                self.success = False
                raise


def main(session, writable, users):
    r = random.randint(0, 0xffffffff)
    request = licensereq.CreateLicenseRequest(r, "ARCHIVER", 0, None)
    license = GetDefaultStore(session).QueryInterface(IID_IECLicense)
    response = license.LicenseAuth(request)
    try:
        licensereq.ProcessLicenseResponse(r, response)
    except RuntimeError:
        print "No valid archiver license found."
        return 2

    global stats
    stats = Stats()
    
    if len(users) == 0:
        users = None
    for store in GetStores(session, users, 0):
        helper = StoreHelper(session, store)
        if not helper.IsArchived():
            stats.storestats.skipped += 1
            continue

        setter = ACLSetter(helper)
        setter.UpdateStoreAccess(writable)
        
        if setter.success:
            stats.storestats.success += 1
        else:
            stats.storestats.failed += 1

    print '**** ACL Set results ****'
    print stats
    return stats.retval
